"""
Copyright (c) 2016, Marcelo Leal
Description: Simple Azure Media Services Python library
License: MIT (see LICENSE.txt file for details)
"""

# restfns - REST functions for amspy

import requests
import json
from .settings import json_acceptformat, xml_acceptformat, batch_acceptformat, charset, dsversion, mdsversion, xmsversion

#Defaults
content_acceptformat = json_acceptformat
acceptformat = json_acceptformat

# do_auth(endpoint, body, access_token)
# do an HTTP POST request for authentication (acquire an access token) and return JSON
def do_auth(endpoint, body):
    headers = {"content-type": "application/x-www-form-urlencoded",
                "Expect" : "100-continue",
                "Accept": acceptformat}
    return requests.post(endpoint, data=body, headers=headers)

# do_get(endpoint, path, access_token)
# do an HTTP GET request and return JSON
def do_get(endpoint, path, access_token):
    headers = {"Content-Type": content_acceptformat,
		"DataServiceVersion": dsversion,
		"MaxDataServiceVersion": mdsversion,
		"Accept": acceptformat,
		"Accept-Charset" : charset,
		"Authorization": "Bearer " + access_token,
		"x-ms-version" : xmsversion}
    body = ''
    response = requests.get(endpoint, headers=headers, allow_redirects=False)
    # AMS response to the first call can be a redirect, 
    # so we handle it here to make it transparent for the caller...
    if (response.status_code == 301):
         redirected_url = ''.join([response.headers['location'], path])
         response = requests.get(redirected_url, data=body, headers=headers)
    return response

# do_put(endpoint, body, access_token)
# do an HTTP PUT request and return JSON
def do_put(endpoint, body, access_token):
    headers = {"content-type": content_acceptformat,
		"Accept": acceptformat,
		"DataServiceVersion": dsversion,
		"MaxDataServiceVersion": mdsversion,
		"Accept-Charset" : charset,
		"x-ms-version" : xmsversion,
		"Expect": "100-continue",
		"Authorization": "Bearer " + access_token}
    return requests.put(endpoint, data=body, headers=headers)

# do_post(endpoint, body, access_token)
# do an HTTP POST request and return JSON
def do_post(endpoint, path, body, access_token, format="json"):
    global acceptformat, content_acceptformat
    if (format == "xml"):
    	content_acceptformat = xml_acceptformat
    	acceptformat = xml_acceptformat + ",application/xml" 
    headers = {"Content-Type": content_acceptformat, 
		"DataServiceVersion": dsversion,
		"MaxDataServiceVersion": mdsversion,
		"Accept": acceptformat,
		"Accept-Charset" : charset,
		"Authorization": "Bearer " + access_token,
		"x-ms-version" : xmsversion}
    response = requests.post(endpoint, data=body, headers=headers, allow_redirects=False)
    # AMS response to the first call can be a redirect, 
    # so we handle it here to make it transparent for the caller...
    if (response.status_code == 301):
         redirected_url = ''.join([response.headers['location'], path])
         response = requests.post(redirected_url, data=body, headers=headers)
    return response

# do_patch(endpoint, path, body, access_token)
# do an HTTP PATCH request and return JSON
def do_patch(endpoint, path, body, access_token):
    headers = {"Content-Type": content_acceptformat, 
		"DataServiceVersion": dsversion,
		"MaxDataServiceVersion": mdsversion,
		"Accept": acceptformat,
		"Accept-Charset" : charset,
		"Authorization": "Bearer " + access_token,
		"x-ms-version" : xmsversion}
    response = requests.patch(endpoint, data=body, headers=headers, allow_redirects=False)
    # AMS response to the first call can be a redirect, 
    # so we handle it here to make it transparent for the caller...
    if (response.status_code == 301):
         redirected_url = ''.join([response.headers['location'], path])
         response = requests.patch(redirected_url, data=body, headers=headers)
    return response

# do_delete(endpoint, access_token)
# do an HTTP DELETE request and return JSON
def do_delete(endpoint, path, access_token):
    headers = {"DataServiceVersion": dsversion,
		"MaxDataServiceVersion": mdsversion,
		"Accept": acceptformat,
		"Accept-Charset" : charset,
		"Authorization": 'Bearer ' + access_token,
		"x-ms-version" : xmsversion}
    response = requests.delete(endpoint, headers=headers, allow_redirects=False)
    # AMS response to the first call can be a redirect, 
    # so we handle it here to make it transparent for the caller...
    if (response.status_code == 301):
         redirected_url = ''.join([response.headers['location'], path])
         response = requests.delete(redirected_url, headers=headers)
    return response

# do_sto_put(endpoint, body, access_token)
# do an HTTP PUT request to the azure storage api and return JSON
def do_sto_put(endpoint, body, content_length, access_token):
    headers = {"Accept": acceptformat,
		"Accept-Charset" : charset,
		"x-ms-blob-type" : "BlockBlob",
		"x-ms-meta-m1": "v1",
		"x-ms-meta-m2": "v2",
		"x-ms-version" : "2015-02-21",
		"Content-Length" : str(content_length)}
    return requests.put(endpoint, data=body, headers=headers)

# do_get_url(endpoint, access_token)
# do an HTTP GET request and return JSON
def do_get_url(endpoint, access_token, flag=True):
    headers = {"Content-Type": content_acceptformat,
		"DataServiceVersion": dsversion,
		"MaxDataServiceVersion": mdsversion,
		"Accept": acceptformat,
		"Accept-Charset" : charset,
		"Authorization": "Bearer " + access_token,
		"x-ms-version" : xmsversion}
    body = ''
    response = requests.get(endpoint, headers=headers, allow_redirects=flag)
    if(flag):
    	if (response.status_code == 301):
    		response = requests.get(response.headers['location'], data=body, headers=headers)
    return response



"""
Copyright (c) 2016, Marcelo Leal
Description: Simple Azure Media Services Python Library
License: MIT (see LICENSE.txt for details)
"""

# settings.py - place to store constants for azurerm

#Endpoints...
ams_auth_endpoint = 'https://wamsprodglobal001acs.accesscontrol.windows.net/v2/OAuth2-13'
ams_rest_endpoint = 'https://media.windows.net/API'

#Headers...
json_acceptformat = "application/json;odata=verbose"
xml_acceptformat = "application/atom+xml"
batch_acceptformat = "multipart/mixed" 
xmsversion = "2.13"
dsversion = "3.0;NetFx"
mdsversion = "3.0;NetFx"
charset = "UTF-8"



"""
Copyright (c) 2016, Marcelo Leal
Description: Simple Azure Media Services Rest Python library
License: MIT (see LICENSE.txt file for details)
"""

# amsrest.py - azurerm functions for the AMS Rest Interface

import urllib
import requests
from .restfns import do_auth, do_get, do_post, do_put, do_delete, do_patch, do_sto_put, do_get_url
from .settings import ams_rest_endpoint, ams_auth_endpoint

# get_access_token(accountname, accountkey)
# get access token with ams
def get_access_token(accountname, accountkey):
    accountkey_encoded = urllib.parse.quote(accountkey, safe='')
    body = "grant_type=client_credentials&client_id=" + accountname + "&client_secret=" + accountkey_encoded + " &scope=urn%3aWindowsAzureMediaServices"
    return do_auth(ams_auth_endpoint, body)

# get_url(access_token)
# get an specific url
def get_url(access_token, endpoint=ams_rest_endpoint, flag=True):
    return do_get_url(endpoint, access_token, flag)

# list_media_asset(access_token, oid)
# list a media asset(s)
def list_media_asset(access_token, oid=""):
    path = '/Assets'
    return helper_list(access_token, oid, path)

# list_content_keys(access_token, oid)
# list the content key(s)
def list_content_key(access_token, oid=""):
    path = '/ContentKeys'
    return helper_list(access_token, oid, path)

# list_media_processor(access_token, oid)
# list the media processor(s)
def list_media_processor(access_token, oid=""):
    path = '/MediaProcessors'
    return helper_list(access_token, oid, path)

# list_asset_accesspolicy(access_token, oid)
# list a asset access policy(ies)
def list_asset_accesspolicy(access_token, oid=""):
    path = '/AccessPolicies'
    return helper_list(access_token, oid, path)

# list_sas_locator(access_token, oid)
# list a sas locator(s)
def list_sas_locator(access_token, oid=""):
    path = '/Locators'
    return helper_list(access_token, oid, path)

# list_media_job(access_token, oid)
# list a media job(s)
def list_media_job(access_token, oid=""):
    path = '/Jobs'
    return helper_list(access_token, oid, path)

# delete_asset_accesspolicy(access_token, oid)
# delete a asset access policy
def delete_asset_accesspolicy(access_token, oid):
    path = '/AccessPolicies'
    return helper_delete(access_token, oid, path)

# delete_sas_locator(access_token, oid)
# delete a sas locator
def delete_sas_locator(access_token, oid):
    path = '/Locators'
    return helper_delete(access_token, oid, path)

# delete_content_key(access_token, oid)
# delete a content key
def delete_content_key(access_token, oid):
    path = '/ContentKeys'
    return helper_delete(access_token, oid, path)

# delete_media_asset(access_token, oid)
# delete a media asset
def delete_media_asset(access_token, oid):
    path = '/Assets'
    return helper_delete(access_token, oid, path)

# create_media_asset(access_token, name, options="0")
# create a media asset
def create_media_asset(access_token, name, options="0"):
    path = '/Assets'
    endpoint = ''.join([ams_rest_endpoint, path])
    body = '{"Name": "' + name + '", "Options": "' + str(options) + '"}'
    return do_post(endpoint, path, body, access_token)

# create_media_assetfile(access_token, parent_asset_id, name, is_primary="false", is_encrypted="false", encryption_scheme="None", encryptionkey_id="None")
# create a media assetfile
def create_media_assetfile(access_token, parent_asset_id, name, is_primary="false", is_encrypted="false", encryption_scheme="None", encryptionkey_id="None"):
    path = '/Files'
    endpoint = ''.join([ams_rest_endpoint, path])
    if (encryption_scheme == "StorageEncryption"):
    	body = '{"IsEncrypted": "' + is_encrypted + '", "EncryptionScheme": "' + encryption_scheme + '", "EncryptionVersion": "' + "1.0" + '", "EncryptionKeyId": "' + encryptionkey_id + '", "IsPrimary": "' + is_primary + '", "MimeType": "video/mp4", "Name": "' + name + '", "ParentAssetId": "' + parent_asset_id + '"}'
    else:
    	body = '{"IsPrimary": "' + is_primary + '", "MimeType": "video/mp4", "Name": "' + name + '", "ParentAssetId": "' + parent_asset_id + '"}'
    return do_post(endpoint, path, body, access_token)

# create_sas_locator(access_token, asset_id, accesspolicy_id)
# create a sas locator
def create_sas_locator(access_token, asset_id, accesspolicy_id):
    path = '/Locators'
    endpoint = ''.join([ams_rest_endpoint, path])
    #body = '{"AccessPolicyId":"' + accesspolicy_id + '", "AssetId":"' + asset_id + '", "StartTime":"' + starttime + '", "Type":1 }'
    body = '{"AccessPolicyId":"' + accesspolicy_id + '", "AssetId":"' + asset_id + '", "Type":1 }'
    return do_post(endpoint, path, body, access_token)

# create_media_task(access_token, processor_id, asset_id, content)
# create a media task
def create_media_task(access_token, processor_id, asset_id, content):
    path = '/Tasks'
    endpoint = ''.join([ams_rest_endpoint, path])

    body = content
    return do_post(endpoint, path, body, access_token)

# create_media_job(access_token, processor_id, asset_id, content)
# create a media job
def create_media_job(access_token, processor_id, asset_id, content):
    path = '/Jobs'
    endpoint = ''.join([ams_rest_endpoint, path])

    body = content
    return do_post(endpoint, path, body, access_token)

# link_content_key(access_token, asset_id, encryptionkey_id)
# link a content key with an asset
def link_content_key(access_token, asset_id, encryptionkey_id, ams_redirected_rest_endpoint):
    path = '/Assets'
    full_path = ''.join([path, "('", asset_id, "')", "/$links/ContentKeys"])
    full_path_encoded = urllib.parse.quote(full_path, safe='')
    endpoint = ''.join([ams_rest_endpoint, full_path_encoded])
    uri = ''.join([ams_redirected_rest_endpoint, 'ContentKeys', "('", encryptionkey_id, "')"])

    body = '{"uri": "' + uri + '"}'
    return do_post(endpoint, full_path_encoded, body, access_token)

# update_media_assetfile(access_token, parent_asset_id, asset_id, content_length, name)
# update a media assetfile
def update_media_assetfile(access_token, parent_asset_id, asset_id, content_length, name):
    path = '/Files'
    full_path = ''.join([path, "('", asset_id, "')"])
    full_path_encoded = urllib.parse.quote(full_path, safe='')
    endpoint = ''.join([ams_rest_endpoint, full_path_encoded])
    body = '{"ContentFileSize": "' + str(content_length) + '", "Id": "' + asset_id + '", "MimeType": "video/mp4", "Name": "' + name + '", "ParentAssetId": "' + parent_asset_id + '"}'
    return do_patch(endpoint, full_path_encoded, body, access_token)

# set_asset_accesspolicy(access_token, duration)
# set a asset access policy
def set_asset_accesspolicy(access_token, duration):
    path = '/AccessPolicies'
    endpoint = ''.join([ams_rest_endpoint, path])
    body = '{"Name": "NewUploadPolicy", "DurationInMinutes": "' + duration + '", "Permissions": "2"}'
    return do_post(endpoint, path, body, access_token)

### Helpers...
# Generic functions not intended for "external" use... 
def helper_list(access_token, oid, path):
    if(oid != ""):
    	path = ''.join([path, "('", oid, "')"])
    endpoint = ''.join([ams_rest_endpoint, path])
    return do_get(endpoint, path, access_token)

def helper_delete(access_token, oid, path):
    full_path = ''.join([path, "('", oid, "')"])
    full_path_encoded = urllib.parse.quote(full_path, safe='')
    endpoint = ''.join([ams_rest_endpoint, full_path_encoded])
    return do_delete(endpoint, full_path_encoded, access_token)

### Aux Funcions...
# These are functions that are intended for "external" use, but are not AMS REST API's...
# Translate the numeric options/encryption of the Asset
def translate_asset_options(nr):
    if (nr == "0"): 
    	return "None"
    if (nr == "1"): 
    	return "StorageEncrypted"
    if (nr == "2"): 
    	return "CommonEncryptionProtected"
    if (nr == "4"): 
    	return "EnvelopeEncryptionProtected"

# Translate the numeric state of the Jobs
def translate_job_state(nr):
    if (nr == "0"): 
    	return "Queued"
    if (nr == "1"): 
    	return "Scheduled"
    if (nr == "2"): 
    	return "Processing"
    if (nr == "3"): 
    	return "Finished"
    if (nr == "4"): 
    	return "Error"
    if (nr == "5"): 
    	return "Canceled"
    if (nr == "6"): 
    	return "Canceling"
# Get specific url
def retrieve_url_content(url):
    return do_get(endpoint, path, access_token)

### Exceptions...
# These, I think, should not be here... ;-)
# upload_block_blob(access_token, endpoint, content, content_length)
# upload a block blob
def upload_block_blob(access_token, endpoint, content, content_length):
    return do_sto_put(endpoint, content, content_length, access_token)

# validate_mp4_asset(access_token, processor_id, asset_id, output_assetname)
# validate a mp4 asset
def validate_mp4_asset(access_token, processor_id, asset_id, output_assetname):
    path = '/Jobs'
    endpoint = ''.join([ams_rest_endpoint, path])

    assets_path = ''.join(["/Assets", "('", asset_id, "')"])
    assets_path_encoded = urllib.parse.quote(assets_path, safe='')
    endpoint_assets = ''.join([ams_rest_endpoint, assets_path_encoded])

    body = '{ \
    		"Name":"ValidateEncodedMP4", \
   		"InputMediaAssets":[{ \
       	  		"__metadata":{ \
       	     			"uri":"' + endpoint_assets + '" \
       	  		} \
     	 	}], \
   		"Tasks":[{ \
       	  		"Configuration":"<?xml version=\\"1.0\\" encoding=\\"utf-8\\"?><taskDefinition xmlns=\\"http://schemas.microsoft.com/iis/media/v4/TM/TaskDefinition#\\"><name>MP4 Preprocessor</name><id>859515BF-9BA3-4BDD-A3B6-400CEF07F870</id><description xml:lang=\\"en\\" /><inputFolder /><properties namespace=\\"http://schemas.microsoft.com/iis/media/V4/TM/MP4Preprocessor#\\" prefix=\\"mp4p\\"><property name=\\"SmoothRequired\\" value=\\"false\\" /><property name=\\"HLSRequired\\" value=\\"true\\" /></properties><taskCode><type>Microsoft.Web.Media.TransformManager.MP4PreProcessor.MP4Preprocessor_Task, Microsoft.Web.Media.TransformManager.MP4Preprocessor, Version=1.0.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35</type></taskCode></taskDefinition>", \
       	  		"MediaProcessorId":"' + processor_id + '", \
       	  		"TaskBody":"<?xml version=\\"1.0\\" encoding=\\"utf-16\\"?><taskBody><inputAsset>JobInputAsset(0)</inputAsset><outputAsset assetCreationOptions=\\"1\\" assetName=\\"' + output_assetname + '\\">JobOutputAsset(0)</outputAsset></taskBody>" \
      		}] \
	}'

    return do_post(endpoint, path, body, access_token)




from setuptools import setup

setup(name='amspy',
      version='0.1.1',
      summary='Simple Python Library for Azure Media Services REST API',
      description='The amspy is a library to provide a simple Azure Media Services REST interface for python. This is a personal project and NOT an official implementation of the Azure Media Services SDK for python. The only purpose of this library is for educational purposes, so people can have an easy way to understand how to interact with cloud REST apis, and learn from the examples provided in this module as well as the debug information available in the logs. Any feedback, comments or bugs, please send directly to the module owner, and go to https://azure.microsoft.com if you are looking for official Microsoft Azure SDKs.',
      url='http://github.com/msleal/amspy',
      author='msleal',
      author_email='msl@eall.com.br',
      license='MIT',
      packages=['amspy'],
      install_requires=[
          'requests',
      ],
      zip_safe=False)



"""
Copyright (c) 2016, Marcelo Leal
Description: Simple Azure Media Services Python library
License: MIT (see LICENSE.txt file for details)
"""

# amspy - library for easy Azure Media Services calls from Python

from .amsrest import get_access_token, create_media_asset, list_media_asset, create_media_assetfile, \
	list_content_key, list_asset_accesspolicy, set_asset_accesspolicy, create_sas_locator, \
	list_sas_locator, list_media_job, list_media_processor, upload_block_blob, update_media_assetfile, \
	delete_sas_locator, delete_asset_accesspolicy, delete_content_key, create_media_job, \
	delete_media_asset, validate_mp4_asset, translate_job_state, translate_asset_options, link_content_key, \
	get_url
